using System.Collections.Generic;
using System.ComponentModel;
using System.Xml.Serialization;

using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

using EQIA.EQIDL;

namespace EQIA
{
    public interface IEQInterfaceService
    {
        /// <summary>
        /// A dictionary of interfaces that are currently loaded keyed by their folder names.
        /// </summary>
        Dictionary<string, IEQInterface> Interfaces { get; }
    }

    public interface IEQInterface
    {
        /// <summary>
        /// The interface this one will fall back on for missing elements.
        /// </summary>
        IEQInterface Parent { get; }

        /// <summary>
        /// A list of elements in the interface.
        /// </summary>
        Dictionary<string, Class> Elements { get; }

        /// <summary>
        /// A list of files in the interface.
        /// </summary>
        Dictionary<string, EQInterfaceFile> Files { get; }

        /// <summary>
        /// The name of the interface's folder.
        /// </summary>
        string Name { get; }

        /// <summary>
        /// The full path to the interface's folder.
        /// </summary>
        string Path { get; }

        /// <summary>
        /// Get an element from this interface.
        /// </summary>
        /// <param name="name">Name of the element to get.</param>
        /// <returns>The element requested or null if it wasn't found in this interface or it's parent.</returns>
        Class GetElement(string name);
    }

    public interface IDrawableSurface : IDrawable
    {
        /// <summary>
        /// Gets a value indicating if the element needs to be redrawn.
        /// </summary>
        [XmlIgnore()]
#if DEBUG
        [Browsable(true), Category("Internal Data")]
#else
        [Browsable(false)]
#endif
        bool IsRequireRedraw { get; }

        /// <summary>
        /// Draw the surface of the element.
        /// </summary>
        void DrawToTexture(SpriteBatch batch);

        /// <summary>
        /// Ensure that the render target for this element is valid.
        /// </summary>
        /// <param name="device"></param>
        void EnsureRenderTarget(GraphicsDevice device);
    }

    public interface IDrawable
    {
        /// <summary>
        /// Draw this element onto the screen.
        /// </summary>
        void Draw(SpriteBatch batch);
    }

    public interface ILoadable
    {
        /// <summary>
        /// Indicates if this element has it's graphical content loaded yet.
        /// </summary>
        [XmlIgnore()]
#if DEBUG
        [Browsable(true), Category("Internal Data")]
#else
        [Browsable(false)]
#endif
        bool IsLoaded { get; }

        /// <summary>
        /// Load graphical content the element requires.
        /// </summary>
        void LoadContent(GraphicsDevice device);

        /// <summary>
        /// Unload graphical content this element requires.
        /// </summary>
        void UnloadContent();
    }

    public interface IUpdateable
    {
        /// <summary>
        /// Performs logic the element requires to function.
        /// </summary>
        void Update(GameTime gameTime);
    }

    public interface IReferer
    {
        /// <summary>
        /// Find the reference elements the implimentor requires.
        /// </summary>
        void ResolveReferences();

        /// <summary>
        /// Check if each of the referenced elements are valid and match the specified 
        /// reference names of the element.
        /// </summary>
        bool ValidateReferences();
    }
}
